import React from "react";
import { Typography, useTheme, Stack, Badge } from "@mui/material";
import {
  Brightness1,
  StopCircle,
  RadioButtonUnchecked,
  HelpOutline,
  PlayCircleFilled,
  PauseCircleFilled
} from "@mui/icons-material";
import Popover from "../../common/Popover";
import { AlertBanner } from "@ess-ics/ce-ui-common";
import LabeledIcon from "../../common/LabeledIcon/LabeledIcon";

const STATUS = {
  active: "active",
  disabled: "disabled",
  alert: "alert",
  inactive: "inactive",
  inactiveAlert: "inactive alert"
};

const SEVERITY = {
  info: "info"
};

function AlertMessagesPopoverContents({ title, alerts }) {
  // Filter out INFO level alerts
  // And for now filter out links on alerts due to issues with
  // focus behavior of popovers
  // Also normalize the type to lowercase since AlertBanner expects lowercase
  const sanitizedAlerts = (
    alerts?.filter((alert) => alert?.type.toLowerCase() !== SEVERITY.info) || []
  ).map((alert) => ({
    ...alert,
    type: alert?.type?.toLowerCase(),
    link: undefined
  }));

  return (
    <Stack
      gap={1}
      sx={{ minWidth: 0, maxWidth: "50vw" }}
    >
      <Typography
        sx={{
          fontWeight: sanitizedAlerts.length > 0 ? "bold" : "normal"
        }}
      >
        {title}
      </Typography>
      {sanitizedAlerts.length > 0
        ? sanitizedAlerts.map((alert, index) => (
            <AlertBanner
              {...alert}
              key={index}
            />
          ))
        : null}
    </Stack>
  );
}

export function IOCStatusIcon({ ioc }) {
  const theme = useTheme();
  let { active, alerts = [], activeDeployment, id } = ioc;
  const alertSeverity = ioc?.alertSeverity?.toLowerCase();

  const iconConfig = {
    [STATUS.active]: {
      title: "Active",
      icon: Brightness1
    },
    [STATUS.disabled]: {
      title: "Disabled",
      icon: StopCircle
    },
    [STATUS.alert]: {
      title: "Alert",
      icon: Brightness1
    },
    [STATUS.inactive]: {
      title: "Inactive",
      icon: RadioButtonUnchecked
    },
    [STATUS.inactiveAlert]: {
      title: "Alert",
      icon: RadioButtonUnchecked
    },
    null: {
      title: "Unknown",
      icon: HelpOutline
    }
  };

  let status = null;

  if (active === null) {
    active = undefined;
  }

  if (
    active &&
    (alertSeverity === undefined || alertSeverity === SEVERITY.info)
  ) {
    // Active status / running
    status = STATUS.active;
  } else if (
    active === false &&
    (alertSeverity === undefined || alertSeverity === SEVERITY.info)
  ) {
    // stopped/paused
    status = STATUS.disabled;
  } else if (
    active !== undefined &&
    alertSeverity !== undefined &&
    alertSeverity !== SEVERITY.info
  ) {
    // warning/error
    status = STATUS.alert;
  } else if (
    (active === undefined || activeDeployment === null) &&
    (alertSeverity === undefined || alertSeverity === SEVERITY.info)
  ) {
    // Inactive status
    status = STATUS.inactive;
  } else if (
    (active === undefined || activeDeployment === null) &&
    alertSeverity !== undefined &&
    alertSeverity !== SEVERITY.info
  ) {
    // inactive with warning/error
    status = STATUS.inactiveAlert;
  } else {
    // unknown fallback state
    status = null;
  }

  const iconTitle = iconConfig[status].title;

  const getLabel = () => {
    if (status === STATUS.alert || status === STATUS.inactiveAlert) {
      const warnings = alerts.filter((alert) => alert.type === "WARNING");
      const errors = alerts.filter((alert) => alert.type === "ERROR");
      const hasWarnings = Boolean(warnings.length);
      const hasErrors = Boolean(errors.length);
      return `${iconTitle} ${hasWarnings ? warnings.length + " warnings" : ""}${
        hasErrors ? errors.length + " errors" : ""
      }`;
    }
    return iconTitle;
  };

  const statusIcon = (
    <LabeledIcon
      label={getLabel()}
      Icon={iconConfig[status].icon}
      labelPosition="none"
    />
  );

  return (
    <div>
      <Popover
        id={`ioc-status-popover-${id}`}
        renderOwningComponent={({
          ariaOwns,
          ariaHasPopup,
          handlePopoverOpen,
          handlePopoverClose
        }) => (
          <div
            aria-owns={ariaOwns}
            aria-haspopup={ariaHasPopup}
            onMouseEnter={handlePopoverOpen}
            onMouseLeave={handlePopoverClose}
            style={{ color: theme.palette.status.icons }}
          >
            {status === STATUS.alert || status === STATUS.inactiveAlert ? (
              <Badge
                badgeContent={"!"}
                color="error"
                sx={{
                  "& .MuiBadge-badge": {
                    minWidth: "17px",
                    height: "17px",
                    fontSize: "0.7rem"
                  }
                }}
              >
                {statusIcon}
              </Badge>
            ) : (
              statusIcon
            )}
          </div>
        )}
        popoverContents={
          <AlertMessagesPopoverContents
            title={iconTitle}
            alerts={alerts}
          />
        }
        anchorOrigin={{
          vertical: "top",
          horizontal: "right"
        }}
      />
    </div>
  );
}

export function CommandTypeIcon({
  type,
  colorStyle = "colors",
  labelPosition = "tooltip"
}) {
  const theme = useTheme();

  const commandTypeColors = {
    start: theme.palette.status.ok.main,
    stop: theme.palette.status.fail.main
  };

  const commandTypeBlack = {
    start: theme.palette.status.icons,
    stop: theme.palette.status.icons
  };

  const colorConfig = {
    black: commandTypeBlack,
    colors: commandTypeColors
  };

  const commandTypeIcons = {
    start: {
      title: "Set to active",
      icon: PlayCircleFilled
    },
    stop: {
      title: "Set to inactive",
      icon: PauseCircleFilled
    }
  };

  const iconStyle = { fill: colorConfig[colorStyle][type.toLowerCase()] };
  const iconTitle = commandTypeIcons[type.toLowerCase()].title;
  const StatusIcon = commandTypeIcons[type.toLowerCase()].icon;

  return (
    <LabeledIcon
      label={iconTitle}
      LabelProps={{ noWrap: true }}
      labelPosition={labelPosition}
      Icon={StatusIcon}
      IconProps={{ style: { iconStyle } }}
    />
  );
}
